:- set(i,4).

:- modeh(1, bind(+pdb)).

%note: aminoacid_id is a single aminoacid identifier that is unique across all pdbs
%:- modeb(*, has_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_aminoacid(+pdb, -aminoacid_id, #aminoacid_name)).
%:- modeb(1, aminoacid_name(+aminoacid_id, #aminoacid_name)).
:- modeb(1, diff_aminoacid(+aminoacid_id, +aminoacid_id),commutative).
%:- modeb(1, diff_aminoacid(+aminoacid_id, +aminoacid_id), commutative).
%:- modeb(1, aminoacid_polarity(+aminoacid_name, #polarity)).
%:- modeb(1, aminoacid_charge(+aminoacid_name, #charge)).
%:- modeb(1, aminoacid_hydropathy(+aminoacid_name, -hydropathy)).

%:- modeb(1, center_coords(+pdb,-coords)). %returns binding site center coordinates
%:- modeb(*, has_atom(+aminoacid_id, #atom_name, -coords)).
%:- modeb(1, dist(+coords,+coords,#dist,#error), commutative).
:- modeb(*, atom_to_center_dist(+aminoacid_id, #atom_name, #dist, #error)).
:- modeb(*, atom_to_atom_dist(+aminoacid_id, +aminoacid_id, #atom_name, #atom_name, #dist, #error), commutative).
%:- modeb(*, ion_to_center_dist(#ion_name, #dist, #error)).
%:- modeb(*, atom_to_ion_dist(+aminoacid_id, #atom_name, #ion_name, #dist, #error)).
%:- modeb(*, has_ion(+pdb, -ion_id,#ion_name,-coord)).

%has_ion(p1BDG,id_1BDG_a617,hoh,coord(23.278,17.343,59.328)).

%ion_to_center_dist(#ion_name, #dist, #error)
ion_to_center_dist(Ion_name, Dist, Tolerance):-
  center_coords(PDB, Coords1),
  has_ion(PDB, _Ion_id, Ion_name, Coords2),
  dist(Coords1, Coords2, Dist, Tolerance).

%atom_to_atom_dist(+aminoacid_id, +aminoacid_id, #atom_name, #atom_name, #dist, #error)
atom_to_ion_dist(Aminoacid_id1, Atom_name, Ion_name, Dist, Tolerance):-
  %Aminoacid_id1\=Aminoacid_id2,
  has_aminoacid(PDB, Aminoacid_id1, _),
  has_atom(Aminoacid_id1, Atom_name, Coords1),
  has_ion(PDB, _Ion_id, Ion_name, Coords2),
  dist(Coords1, Coords2, Dist, Tolerance).

%atom_to_center_dist(+aminoacid_id, #atom_name, #dist, #error)
atom_to_center_dist(Aminoacid_id, Atom_name, Dist, Tolerance):-
  has_aminoacid(PDB, Aminoacid_id, _),
  has_atom(Aminoacid_id, Atom_name, Coords1),
  center_coords(PDB, Coords2),
  dist(Coords1, Coords2, Dist, Tolerance).

%atom_to_atom_dist(+aminoacid_id, +aminoacid_id, #atom_name, #atom_name, #dist, #error)
atom_to_atom_dist(Aminoacid_id1, Aminoacid_id2, Atom_name1, Atom_name2, Dist, Tolerance):-
  %Aminoacid_id1\=Aminoacid_id2,
  has_aminoacid(PDB, Aminoacid_id1, _),
  has_atom(Aminoacid_id1, Atom_name1, Coords1),
  has_aminoacid(PDB, Aminoacid_id2, _),
  has_atom(Aminoacid_id2, Atom_name2, Coords2),
  dist(Coords1, Coords2, Dist, Tolerance).

% these require i=4
%:- modeb(1, gteq(+hydropathy,#hydropathy)).
%:- modeb(1, lteq(+hydropathy,#hydropathy)).

%:- [aminoacids].      % includes: has_aminoacid/2
:- [aminoacids3].     % includes: has_aminoacid/3
%:- [amin_names].      % includes: aminoacid_name/2
%:- [amin_properties]. % includes: aminoacid_polarity/2, aminoacid_charge/2, aminoacid_hydropathy/2
:- [atoms].           % includes: has_atom/3
:- ['../common/center_coords'].   % includes: center_coords/2
:- ['../common/dist'].            % includes: dist/4, gteq/2, lteq/2
:- [has_ion].

diff_aminoacid(A, B):-
  A\=B.
/*
has_aminoacid(Pdb, Aminoacid_id, Aminoacid_name):-
  has_aminoacid(Pdb, Aminoacid_id),
  aminoacid_name(Aminoacid_id, Aminoacid_name).
*/

%:- [amin_groups].
/*
:- modeb(*, has_aliphatic_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_aromatic_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_positive_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_negative_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_amide_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_acide_and_amide_aminoacid(+pdb, -aminoacid_id)).
:- modeb(*, has_polar_aminoacid(+pdb, -aminoacid_id)).
*/
has_aliphatic_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  aliphatic(AminName).

has_aromatic_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  aromatic(AminName).

has_positive_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  positive(AminName).

has_negative_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  negative(AminName).

has_amide_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  amide(AminName).

has_acide_and_amide_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  acide_and_amide(AminName).

has_polar_aminoacid(PDB_ID, AminID):-
  has_aminoacid(PDB_ID, AminID, AminName),
  polar(AminName).
