#!/bin/sh
#
# The purpose of this Linux shell script is to run aleph with 10 fold cross validation
# Author: Jose Santos <jcas81@gmail.com> 19 June 2008
#
# It assumes the following files are in the root directory:
#   aleph.pl
#   problem.pl 
#   run (this file)
#
#  It is also required for the problems to solve to be in the directories in the root
#  The structure of each of these directories is:
#    problem_dir
#        background prolog files (namely a modes.b must exist)
#        a sub-directory folds with files train#.? where # varies from 1..10 and # is f for
#        positive examples and n for negative examples
#
#  The output goes to file log.txt. For the stdout times and accuracies are printed

if test -z $1
then
  echo First argument not defined. Should be directory name.
  exit
fi
LOG_FILE=$1'/aleph_log.txt'
echo > ${LOG_FILE}
#D=\'$1/folds/
D=\'common/folds/
A=set_prolog_flag\(single_var_warnings,off\), # to disable singleton warnings
YAP=yap
for i in 1 2 3 4 5 6 7 8 9 10
do
  echo Fold $i
  T=$D\train$i\'  # training data without the extension
  B=read_background\(\'$1\/hexose\'\),read_examples\($T,$T\),
  C=set\(test_pos,$D\test$i.f\'\),set\(test_neg,$D\test$i.n\'\),induce,program_stats,halt.
  echo $A$B$C| $YAP -q -l run_aleph_auto.pl >> ${LOG_FILE}
done
echo Using all data for training
V=read_background\(\'$1\/hexose\'\),read_examples\($D\train_all\',$D\train_all\'\),induce,program_stats,halt.
echo $A$V| $YAP -q -l run_aleph_auto.pl >> ${LOG_FILE}
grep Accuracy ${LOG_FILE} | gawk -F= '{ print $2 }'
grep cputime ${LOG_FILE} | gawk -F: '{ print $2 }'
